"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DialogflowESResponse = void 0;
const dialogflow_1 = require("@google-cloud/dialogflow");
const typebox_1 = require("@sinclair/typebox");
// TODO: explore breaking out schemas and using `Type.Ref`
exports.DialogflowESResponse = typebox_1.Type.Object({
    fulfillmentText: typebox_1.Type.Optional(typebox_1.Type.String()),
    fulfillmentMessages: typebox_1.Type.Array(typebox_1.Type.Union([
        typebox_1.Type.Object({
            text: typebox_1.Type.Object({
                text: typebox_1.Type.Array(typebox_1.Type.String()),
            }),
        }),
        typebox_1.Type.Object({
            image: typebox_1.Type.Object({
                imageUri: typebox_1.Type.Optional(typebox_1.Type.String()),
                accessibilityText: typebox_1.Type.Optional(typebox_1.Type.String()),
            }),
        }),
        typebox_1.Type.Object({
            quickReplies: typebox_1.Type.Object({
                title: typebox_1.Type.Optional(typebox_1.Type.String()),
                quickReplies: typebox_1.Type.Optional(typebox_1.Type.Array(typebox_1.Type.String())),
            }),
        }),
        typebox_1.Type.Object({
            card: typebox_1.Type.Object({
                title: typebox_1.Type.Optional(typebox_1.Type.String()),
                subtitle: typebox_1.Type.Optional(typebox_1.Type.String()),
                imageUri: typebox_1.Type.Optional(typebox_1.Type.String()),
                buttons: typebox_1.Type.Optional(typebox_1.Type.Array(typebox_1.Type.Object({
                    text: typebox_1.Type.Optional(typebox_1.Type.String()),
                    postback: typebox_1.Type.Optional(typebox_1.Type.String()),
                }))),
            }),
        }),
        typebox_1.Type.Object({
            payload: typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Any()),
        }),
        typebox_1.Type.Object({
            simpleResponses: typebox_1.Type.Object({
                simpleResponses: typebox_1.Type.Array(typebox_1.Type.Union([
                    typebox_1.Type.Object({
                        textToSpeech: typebox_1.Type.String(),
                        displayText: typebox_1.Type.Optional(typebox_1.Type.String()),
                    }),
                    typebox_1.Type.Object({
                        ssml: typebox_1.Type.String(),
                        displayText: typebox_1.Type.Optional(typebox_1.Type.String()),
                    }),
                ])),
            }),
        }),
        typebox_1.Type.Object({
            basicCard: typebox_1.Type.Object({
                title: typebox_1.Type.Optional(typebox_1.Type.String()),
                subtitle: typebox_1.Type.Optional(typebox_1.Type.String()),
                formattedText: typebox_1.Type.Optional(typebox_1.Type.String()),
                image: typebox_1.Type.Optional(typebox_1.Type.Object({
                    imageUri: typebox_1.Type.Optional(typebox_1.Type.String()),
                    accessibilityText: typebox_1.Type.Optional(typebox_1.Type.String()),
                })),
                buttons: typebox_1.Type.Optional(typebox_1.Type.Array(typebox_1.Type.Object({
                    text: typebox_1.Type.Optional(typebox_1.Type.String()),
                    postback: typebox_1.Type.Optional(typebox_1.Type.String()),
                }))),
            }),
        }),
        typebox_1.Type.Object({
            suggestions: typebox_1.Type.Object({
                suggestions: typebox_1.Type.Array(typebox_1.Type.Object({
                    title: typebox_1.Type.String(),
                })),
            }),
        }),
        // TODO: finish adding additional types
        // https://cloud.google.com/dialogflow/es/docs/reference/rest/v2/projects.agent.intents#Intent.Message
        typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Any()),
    ])),
    endInteraction: typebox_1.Type.Optional(typebox_1.Type.Boolean()),
    source: typebox_1.Type.Optional(typebox_1.Type.String()),
    payload: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Any())])),
    outputContexts: typebox_1.Type.Optional(typebox_1.Type.Array(typebox_1.Type.Object({
        name: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.String()])),
        lifespanCount: typebox_1.Type.Optional(typebox_1.Type.Any()),
        parameters: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Any())])),
    }))),
    followupEventInput: typebox_1.Type.Optional(typebox_1.Type.Union([
        typebox_1.Type.Null(),
        typebox_1.Type.Object({
            name: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.String()])),
            parameters: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Any())])),
            languageCode: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.String()])),
        }),
    ])),
    sessionEntityTypes: typebox_1.Type.Optional(typebox_1.Type.Union([
        typebox_1.Type.Null(),
        typebox_1.Type.Array(typebox_1.Type.Object({
            name: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.String()])),
            entityOverrideMode: typebox_1.Type.Optional(typebox_1.Type.Union([
                typebox_1.Type.Null(),
                typebox_1.Type.Enum(dialogflow_1.protos.google.cloud.dialogflow.v2beta1.SessionEntityType.EntityOverrideMode),
            ])),
            entities: typebox_1.Type.Optional(typebox_1.Type.Union([
                typebox_1.Type.Null(),
                typebox_1.Type.Array(typebox_1.Type.Object({
                    value: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.String()])),
                    synonyms: typebox_1.Type.Optional(typebox_1.Type.Union([typebox_1.Type.Null(), typebox_1.Type.Array(typebox_1.Type.String())])),
                })),
            ])),
        })),
    ])),
});
