import { protos } from '@google-cloud/dialogflow';
import { Static, Type } from '@sinclair/typebox';

// TODO: explore breaking out schemas and using `Type.Ref`
export const DialogflowESResponse = Type.Object({
  fulfillmentText: Type.Optional(Type.String()),
  fulfillmentMessages: Type.Array(
    Type.Union([
      Type.Object({
        text: Type.Object({
          text: Type.Array(Type.String()),
        }),
      }),
      Type.Object({
        image: Type.Object({
          imageUri: Type.Optional(Type.String()),
          accessibilityText: Type.Optional(Type.String()),
        }),
      }),
      Type.Object({
        quickReplies: Type.Object({
          title: Type.Optional(Type.String()),
          quickReplies: Type.Optional(Type.Array(Type.String())),
        }),
      }),
      Type.Object({
        card: Type.Object({
          title: Type.Optional(Type.String()),
          subtitle: Type.Optional(Type.String()),
          imageUri: Type.Optional(Type.String()),
          buttons: Type.Optional(
            Type.Array(
              Type.Object({
                text: Type.Optional(Type.String()),
                postback: Type.Optional(Type.String()),
              })
            )
          ),
        }),
      }),
      Type.Object({
        payload: Type.Record(Type.String(), Type.Any()),
      }),
      Type.Object({
        simpleResponses: Type.Object({
          simpleResponses: Type.Array(
            Type.Union([
              Type.Object({
                textToSpeech: Type.String(),
                displayText: Type.Optional(Type.String()),
              }),
              Type.Object({
                ssml: Type.String(),
                displayText: Type.Optional(Type.String()),
              }),
            ])
          ),
        }),
      }),
      Type.Object({
        basicCard: Type.Object({
          title: Type.Optional(Type.String()),
          subtitle: Type.Optional(Type.String()),
          formattedText: Type.Optional(Type.String()),
          image: Type.Optional(
            Type.Object({
              imageUri: Type.Optional(Type.String()),
              accessibilityText: Type.Optional(Type.String()),
            })
          ),
          buttons: Type.Optional(
            Type.Array(
              Type.Object({
                text: Type.Optional(Type.String()),
                postback: Type.Optional(Type.String()),
              })
            )
          ),
        }),
      }),
      Type.Object({
        suggestions: Type.Object({
          suggestions: Type.Array(
            Type.Object({
              title: Type.String(),
            })
          ),
        }),
      }),
      // TODO: finish adding additional types
      // https://cloud.google.com/dialogflow/es/docs/reference/rest/v2/projects.agent.intents#Intent.Message
      Type.Record(Type.String(), Type.Any()),
    ])
  ),
  endInteraction: Type.Optional(Type.Boolean()),
  source: Type.Optional(Type.String()),
  payload: Type.Optional(Type.Union([Type.Null(), Type.Record(Type.String(), Type.Any())])),
  outputContexts: Type.Optional(
    Type.Array(
      Type.Object({
        name: Type.Optional(Type.Union([Type.Null(), Type.String()])),
        lifespanCount: Type.Optional(Type.Any()), // https://github.com/nestjs/swagger/issues/1162
        parameters: Type.Optional(Type.Union([Type.Null(), Type.Record(Type.String(), Type.Any())])),
      })
    )
  ),
  followupEventInput: Type.Optional(
    Type.Union([
      Type.Null(),
      Type.Object({
        name: Type.Optional(Type.Union([Type.Null(), Type.String()])),
        parameters: Type.Optional(Type.Union([Type.Null(), Type.Record(Type.String(), Type.Any())])),
        languageCode: Type.Optional(Type.Union([Type.Null(), Type.String()])),
      }),
    ])
  ),
  sessionEntityTypes: Type.Optional(
    Type.Union([
      Type.Null(),
      Type.Array(
        Type.Object({
          name: Type.Optional(Type.Union([Type.Null(), Type.String()])),
          entityOverrideMode: Type.Optional(
            Type.Union([
              Type.Null(),
              Type.Enum<Record<string, any>>(
                protos.google.cloud.dialogflow.v2beta1.SessionEntityType.EntityOverrideMode
              ),
            ])
          ),
          entities: Type.Optional(
            Type.Union([
              Type.Null(),
              Type.Array(
                Type.Object({
                  value: Type.Optional(Type.Union([Type.Null(), Type.String()])),
                  synonyms: Type.Optional(Type.Union([Type.Null(), Type.Array(Type.String())])),
                })
              ),
            ])
          ),
        })
      ),
    ])
  ),
});

export type DialogflowESResponse = Static<typeof DialogflowESResponse>;
