import { Body, Controller, Param, Post, UseFilters, Version } from '@nestjs/common';
import { ApiBody, ApiOkResponse, ApiOperation, ApiTags } from '@nestjs/swagger';
import { HttpErrorFilter } from '@voiceflow/runtime-sdk-nestjs';
import { TypeboxValidationPipe } from '@voiceflow/typebox-validator-nestjs';

import { DialogflowESRequest } from './dto/DialogflowESRequest.dto';
import { DialogflowESResponse } from './dto/DialogflowESResponse.dto';
import { InteractService } from './interact.service';
import { InteractRequestTransformer } from './transformers/request.transformer';

@Controller('interact')
@UseFilters(HttpErrorFilter)
@ApiTags('Conversation')
export class InteractController {
  public constructor(
    private readonly interactService: InteractService,
    private readonly requestTransformer: InteractRequestTransformer
  ) {}

  @Version('1beta1')
  @Post(':versionID')
  @ApiOperation({ summary: 'Interact' })
  @ApiBody({ schema: DialogflowESRequest })
  @ApiOkResponse({ schema: DialogflowESResponse })
  async interact(
    @Param('versionID') versionID: string,
    @Body(TypeboxValidationPipe(DialogflowESRequest)) body: DialogflowESRequest
  ): Promise<DialogflowESResponse> {
    const request = this.requestTransformer.transform(body);

    return this.interactService.handle({
      ...request,
      versionID,
    });
  }
}
