import { protos } from '@google-cloud/dialogflow';
import { Injectable } from '@nestjs/common';
import { IntentAction, RuntimeInteractRequest } from '@voiceflow/runtime-sdk';

import { DialogflowESRequest } from '../dto/DialogflowESRequest.dto';

@Injectable()
export class InteractRequestTransformer {
  private getGoogleProtobufValue(value: protos.google.protobuf.IValue) {
    if (value.boolValue != null) return value.boolValue;
    if (value.listValue != null) return value.listValue.values?.map((value) => this.getGoogleProtobufValue(value));
    if (value.nullValue != null) return null;
    if (value.numberValue != null) return value.numberValue;
    if (value.stringValue != null) return value.stringValue;
    if (value.structValue != null)
      return Object.fromEntries(
        Object.entries(value.structValue).map(([key, value]) => [key, this.getGoogleProtobufValue(value)])
      );
    return null;
  }

  public transform(request: DialogflowESRequest): RuntimeInteractRequest {
    const slots = Object.entries(request.queryResult.parameters ?? {}) as [string, protos.google.protobuf.IValue][];

    const action: IntentAction = {
      type: 'intent',
      payload: {
        query: request.queryResult.queryText,
        intent: {
          name: request.queryResult.intent.displayName,
        },
        entities: slots.map(([key, value]) => {
          return {
            name: key,
            value: JSON.stringify(this.getGoogleProtobufValue(value)),
          };
        }),
        confidence: request.queryResult.intentDetectionConfidence ?? undefined,
      },
    };

    // TODO: In google-runtime, we store the entire path in storage.
    // We will have to decode this on general-runtime, then possibly migrate existing sessions that reference the original request.session.
    const sessionID = encodeURIComponent(request.session);

    return {
      action,
      sessionID,
    };
  }
}
